@@ -1,4 +1,4 @@
-import contextlib, datetime, itertools, os, pathlib, random, re, subprocess, sys, time, uuid
+import contextlib, datetime, itertools, methodtools, os, pathlib, random, re, subprocess, sys, time, uuid
 if sys.platform != "cygwin":
   import psutil
 
@@ -14,18 +14,33 @@ def rm_missing_ok(path):
 def SLURM_JOBID():
   return os.environ.get("SLURM_JOBID", None)
 
+def cpuid():
+  node = uuid.getnode()
+  #least significant bit of the first octet is not set --> this is a hardware address
+  if not node & 2**40:
+    return node
+
+  #otherwise getnode gave us a random number!
+  for filename in "/etc/machine-id", "/var/lib/dbus/machine-id":
+    try:
+      with open(filename) as f:
+        machineid = f.read().strip()
+    except FileNotFoundError:
+      continue
+    return uuid.UUID(machineid).int
+
+  raise ValueError("Couldn't find a cpuid using any of the methods we know about")
+
 def jobinfo():
   if SLURM_JOBID() is not None:
     return "SLURM", 0, SLURM_JOBID()
-  return sys.platform, uuid.getnode(), os.getpid()
+  return sys.platform, cpuid(), os.getpid()
 
 class JobLock(object):
   defaultcorruptfiletimeout = None
 
   def __init__(self, filename, *, outputfiles=[], checkoutputfiles=True, inputfiles=[], checkinputfiles=True, corruptfiletimeout=None, mkdir=False):
     self.filename = pathlib.Path(filename)
-    self.fd = self.f = None
-    self.bool = False
     self.outputfiles = [pathlib.Path(_) for _ in outputfiles]
     self.inputfiles = [pathlib.Path(_) for _ in inputfiles]
     self.checkoutputfiles = outputfiles and checkoutputfiles
@@ -35,6 +50,12 @@ class JobLock(object):
       corruptfiletimeout = self.defaultcorruptfiletimeout
     self.corruptfiletimeout = corruptfiletimeout
     self.mkdir = mkdir
+    self.__reset()
+
+  def __reset(self):
+    self.fd = self.f = None
+    self.bool = False
+    self.__inputsexist = self.__outputsexist = self.__oldjobinfo = None
 
   @property
   def wouldbevalid(self):
@@ -61,6 +82,21 @@ class JobLock(object):
       if exceptions: raise
       return None, None, None
 
+  @methodtools.lru_cache()
+  @property
+  def outputsexist(self):
+    return self.__outputsexist
+
+  @methodtools.lru_cache()
+  @property
+  def inputsexist(self):
+    return self.__inputsexist
+
+  @methodtools.lru_cache()
+  @property
+  def oldjobinfo(self):
+    return self.__oldjobinfo
+
   def __open(self):
     self.fd = os.open(self.filename, os.O_CREAT | os.O_EXCL | os.O_WRONLY)
 
@@ -101,11 +137,15 @@ class JobLock(object):
           break
 
   def __enter__(self):
-    removed_failed_job = False
-    if self.checkoutputfiles and all(_.exists() for _ in self.outputfiles) and not self.filename.exists():
-      return None
-    if self.checkinputfiles and not all(_.exists() for _ in self.inputfiles):
-      return None
+    self.removed_failed_job = removed_failed_job = False
+    if self.checkoutputfiles and not self.filename.exists():
+      self.__outputsexist = {_: _.exists() for _ in self.outputfiles}
+      if all(self.outputsexist.values()):
+        return self
+    if self.checkinputfiles:
+      self.__inputsexist = {_: _.exists() for _ in self.inputfiles}
+      if not all(self.inputsexist.values()):
+        return self
     if self.mkdir:
       self.filename.parent.mkdir(parents=True, exist_ok=True)
     try:
@@ -117,15 +157,17 @@ class JobLock(object):
       #job failed at the same time, and one of them could remove
       #the lock created by the other one
       with JobLock(self.iterative_lock_filename, corruptfiletimeout=self.corruptfiletimeout) as iterative_lock:
-        if not iterative_lock: return None
+        if not iterative_lock: return self
         try:
-          oldjobinfo = self.runningjobinfo(exceptions=True)
-        except (IOError, OSError):
+          self.__oldjobinfo = self.runningjobinfo(exceptions=True)
+        except (IOError, OSError) as e:
+          self.__oldjobinfo = e
           try:
             self.__open()
           except FileExistsError:
-            return None
-        except ValueError:
+            return self
+        except ValueError as e:
+          self.__oldjobinfo = e
           if self.corruptfiletimeout is not None:
             modified = datetime.datetime.fromtimestamp(self.filename.stat().st_mtime)
             now = datetime.datetime.now()
@@ -137,23 +179,23 @@ class JobLock(object):
               try:
                 self.__open()
               except FileExistsError:
-                return None
+                return self
             else:
-              return None
+              return self
           else:
-            return None
+            return self
         else:
-          if jobfinished(*oldjobinfo):
+          if jobfinished(*self.oldjobinfo):
             for outputfile in self.outputfiles:
               rm_missing_ok(outputfile)
             rm_missing_ok(self.filename)
-            removed_failed_job = True
+            self.removed_failed_job = removed_failed_job = True
             try:
               self.__open()
             except FileExistsError:
-              return None
+              return self
           else:
-            return None
+            return self
 
     self.f = os.fdopen(self.fd, 'w')
 
@@ -180,12 +222,20 @@ class JobLock(object):
       self.clean_up_iterative_locks()
       #remove this lock file
       rm_missing_ok(self.filename)
-    self.fd = self.f = None
-    self.bool = self.removed_failed_job = False
+    self.__reset()
 
   def __bool__(self):
     return self.bool
 
+  @property
+  def debuginfo(self):
+    return {
+      "outputsexist": self.outputsexist,
+      "inputsexist": self.inputsexist,
+      "oldjobinfo": self.oldjobinfo,
+      "removed_failed_job": self.removed_failed_job,
+    }
+
 def jobfinished(jobtype, cpuid, jobid):
   if jobtype == "SLURM":
     try:
